const jsx = require('babel-plugin-syntax-jsx');

function hmrImport(t) {
    const specifiers = [t.importDefaultSpecifier(t.identifier('reactTransform'))];
    return t.importDeclaration(specifiers, t.stringLiteral('react-transform-hmr'));
}

function wrapComponent(t, className) {
    const declares = [
        t.variableDeclarator(
            t.identifier(className),
            t.callExpression(t.identifier('wrapComponent'), [
                t.stringLiteral(className),
                t.identifier('__' + className)
            ])
        )
    ];

    return t.variableDeclaration('const', declares);
}

/* function wrapComponent(id, Component) {
    const t = reactTransform({
        components: {
            [id]: {
                displayName: id
            }
        },
        locals: [module],
        imports: [React]
    });
    return t(Component, id);
}
 */

function declareWrapComponent(t, className) {
    const displayNameProp = t.objectProperty(t.identifier('displayName'), t.identifier('id'));
    const displayNameObject = t.objectExpression([displayNameProp]);

    const idProp = t.objectProperty(t.identifier('[id]'), displayNameObject);
    const idObject = t.objectExpression([idProp]);

    const localsProp = t.objectProperty(t.identifier('locals'), t.identifier('[module]'));
    const reactProp = t.objectProperty(t.identifier('imports'), t.identifier('[React]'));

    const componentsProp = t.objectProperty(t.identifier('components'), idObject);
    const componentsObject = t.objectExpression([componentsProp, localsProp, reactProp]);

    const declares = [
        t.variableDeclarator(t.identifier('t'), t.callExpression(t.identifier('reactTransform'), [componentsObject]))
    ];
    const declareT = t.variableDeclaration('const', declares);

    const callT = t.callExpression(t.identifier('t'), [t.identifier('Component'), t.identifier('id')]);
    const returnStatement = t.returnStatement(callT);

    const params = [t.identifier('id'), t.identifier('Component')];
    const body = t.blockStatement([declareT, returnStatement]);
    return t.functionDeclaration(t.identifier('wrapComponent'), params, body);
}

function renderFunction(t, arrowFunction) {
    const properties = arrowFunction.params[0].properties.map(function(prop) {
        return prop.key;
    });

    const returnStatement = t.returnStatement(t.parenthesizedExpression(arrowFunction.body));

    const declares = properties.map(function(p) {
        return t.variableDeclarator(
            p,
            t.memberExpression(t.memberExpression(t.identifier('this'), t.identifier('props')), p)
        );
    });
    const destructProps = t.variableDeclaration('const', declares);

    const renderBody = t.blockStatement([destructProps, returnStatement]);
    return t.classMethod('method', t.identifier('render'), [], renderBody);
}

module.exports = function({ Plugin, types: t }) {
    let alreadyImported = false;

    const visitor = {};
    const arrowVisitor = function(state) {
        return function(path) {
            const filename = state.file.opts.filename;
            if (filename.indexOf('node_modules') > -1) return;

            if (filename !== '/Users/bvic23/Sites/Yojee/driver-react-native/storybook/stories/Button/index.android.js')
                return;
            console.warn('#############################------->', state.file.opts.filename);
            console.warn('AAAAAAA------->', path.node.type);

            const arrowFunction = path.node;

            const id = path.parentPath.node.id;
            const className = id ? id.name : '__Clazz__';
            const isDeclaredAsConstant = id !== undefined;

            const renderMethod = renderFunction(t, arrowFunction);
            const classBody = t.classBody([renderMethod]);
            const classDeclaration = t.classDeclaration(
                t.identifier('__' + className),
                t.identifier('require("react").Component'),
                classBody,
                []
            );

            if (!alreadyImported) {
                alreadyImported = true;
                path.parentPath.parentPath.insertBefore(hmrImport(t));
            }

            if (isDeclaredAsConstant) {
                path.parentPath.parentPath.replaceWith(classDeclaration);
                path.parentPath.parentPath.insertAfter(wrapComponent(t, className));
                path.parentPath.parentPath.insertBefore(declareWrapComponent(t, className));
            } else {
                const exporDefault = t.exportDefaultDeclaration(classDeclaration);
                path.parentPath.replaceWith(exporDefault);
            }
        };
    };

    visitor.Program = function(path, state) {
        path.traverse({ ArrowFunctionExpression: arrowVisitor(state) });
    };

    return {
        inherits: jsx,
        visitor
    };
};
